# Generated by fpm
# Hello packaging friend!
#
# If you find yourself using this 'fpm --edit' feature frequently, it is
# a sign that fpm is missing a feature! I welcome your feature requests!
# Please visit the following URL and ask for a feature that helps you never
# need to edit this file again! :)
#   https://github.com/jordansissel/fpm/issues
# ------------------------------------------------------------------------
#
pkgname = <%= name %>
<% if !epoch.nil? -%>
pkgver = <%= epoch %>:<%= version %>-<%= iteration %>
<% else -%>
pkgver = <%= version %>-<%= iteration %>
<% end -%>
pkgdesc = <%= description %>
<% if !url.nil? and !url.empty? -%>
url = <%= url %>
<% else -%>
url = http://nourlgiven.example.com/
<% end -%>
builddate = <%= builddate %>
packager = <%= maintainer %>
size = <%= size %>
arch = <%= architecture %>
license = <%= license %>
<% replaces.each do |repl| -%>
replaces = <%= repl %>
<% end -%>
group = <%= category %>
<% conflicts.each do |confl| -%>
conflict = <%= confl %>
<% end -%>
<% provides.each do |prov| -%>
provides = <%= prov %>
<% end -%>
<% config_files.each do |conf| -%>
<%   conf.gsub(/^\/+/, "") -%>
backup = <%= conf %>
<% end -%>
<% dependencies.each do |req| -%>
depend = <%= req %>
<% end -%>
<% attributes.fetch(:pacman_opt_depends, []).each do |opt| -%>
optdepend = <%= opt %>
<% end -%>
