# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

require "erb"

module LogStash::PluginManager
  class RenderContext
    def initialize(options = {})
      options.each do |name, value|
        define_singleton_method(name) { value }
      end
    end

    def get_binding
      binding()
    end

    def classify(klass_name)
      klass_name.split(/-|_/).map { |e| e.capitalize }.join("")
    end

  end
end
