<%#
# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
%>
<%- @spec.path_parts.each do |name, _| %>
  <%- unless @spec.method_name == 'get_field_mapping' && name == 'fields' %>
    <%= "_#{name}" %> = arguments.delete(:<%=name %>)
  <%- end -%>
<%- end -%>

method = <%= @spec.http_method %>
<%- if @spec.method_name == 'termvectors' %>
  arguments.delete(:endpoint)
<%- end -%>
path   = <%= @spec.http_path %>
<%- if !@spec.params.empty? || needs_ignore_404?(@spec.endpoint_name) || needs_complex_ignore_404?(@spec.endpoint_name)-%>
  params = Utils.process_params(arguments)
<%- else -%>
  params = {}
<%- end -%>
<%- @spec.specific_params.each do |param| -%>
  <%= param %>
<%- end -%>
