# AUTOGENERATED BY THE GRADLE SCRIPT. DO NOT EDIT.

require 'jar_dependencies'
require_jar('io.netty', 'netty-handler', '4.1.131.Final')
require_jar('org.javassist', 'javassist', '3.24.0-GA')
require_jar('io.netty', 'netty-transport-native-unix-common', '4.1.131.Final')
require_jar('io.netty', 'netty-codec', '4.1.131.Final')
require_jar('io.netty', 'netty-transport', '4.1.131.Final')
require_jar('io.netty', 'netty-resolver', '4.1.131.Final')
require_jar('io.netty', 'netty-buffer', '4.1.131.Final')
require_jar('io.netty', 'netty-common', '4.1.131.Final')
require_jar('org.logstash.beats', 'logstash-input-beats', '7.0.7')
