# encoding: utf-8
require 'logstash/devutils/rspec/shared_examples'
require "logstash/devutils/rspec/spec_helper"
require "logstash/inputs/<%= plugin_name %>"

describe LogStash::Inputs::<%= classify(plugin_name) %> do

  it_behaves_like "an interruptible input plugin" do
    let(:config) { { "interval" => 100 } }
  end

end
