<%#
# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
%>
<%- if @spec.documentation['description'] -%>
  # <%= @spec.documentation['description'].gsub("\n", "\n  # ") -%>
<%- else %>
  # TODO: Description
<%- end %>
<%= stability_doc_helper(@spec.stability) -%>
#
<%- unless @spec.path_parts.nil? || @spec.path_parts.empty? %><%# URL parts -%>
  <%- @spec.path_parts.each do |name, info| -%>
    <%= docs(name, info) -%>
  <%- end -%>
<%- end -%><%# Body -%>
<%# URL parameters -%>
<%- @spec.params.each do |name, info| -%>
  <%= docs(name, info) unless (!@spec.path_parts.empty? && @spec.path_parts.keys.include?(name)) -%>
<%- end -%>
# @option arguments [Hash] :headers Custom HTTP headers
<%- if @spec.body -%>
  <%- if @spec.method_name == 'bulk' -%>
    <%= bulk_doc_helper(@spec.body['description']) -%>
  <%- else -%>
    <%= '# @option arguments [Hash] :body ' + (@spec.body['description'] ?
                                               @spec.body['description'].strip : 'TODO: Description') +
                                              (@spec.body['required'] ? ' (*Required*)' : '') + "\n"
    -%>
  <%- end -%>
<%- end -%>
<% if @spec.deprecation_note -%>
  #
  # *Deprecation notice*:
  # <%=  @spec.deprecation_note['description'] %>
  # Deprecated since version <%=  @spec.deprecation_note['version'] %>
  #
<% end -%>
#
<%# Documentation link -%>
# @see <%= @spec.documentation['url'] ? Elasticsearch::API::FilesHelper.documentation_url(@spec.documentation['url']) : "[TODO]" %>
#
