# Licensed to Elasticsearch B.V. under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Elasticsearch B.V. licenses this file to you under
# the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

require "logstash/pipeline_action/base"
require "logstash/pipeline_action/create"
require "logstash/pipeline_action/stop"
require "logstash/pipeline_action/reload"
require "logstash/pipeline_action/delete"
require "logstash/pipeline_action/stop_and_delete"

module LogStash module PipelineAction
  ORDERING = {
    LogStash::PipelineAction::Create => 100,
    LogStash::PipelineAction::Reload => 200,
    LogStash::PipelineAction::Stop => 300,
    LogStash::PipelineAction::StopAndDelete => 350,
    LogStash::PipelineAction::Delete => 400
  }
end end
