# encoding: UTF-8

# This file contains data derived from the IANA Time Zone Database
# (https://www.iana.org/time-zones).

module TZInfo
  module Data
    module Definitions
      module GMT__p__0
        include TimezoneDefinition
        
        linked_timezone 'GMT+0', 'Etc/GMT'
      end
    end
  end
end
