<settings xmlns="http://maven.apache.org/SETTINGS/1.0.0"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xsi:schemaLocation="http://maven.apache.org/SETTINGS/1.0.0
  https://maven.apache.org/xsd/settings-1.0.0.xsd">
  <proxies>
    <% proxies.each_with_index do |proxy, idx| %>
    <proxy>
      <id><%=proxy.host%>.<%=idx%></id>
      <active>true</active>
      <protocol><%=proxy.protocol%></protocol>
      <host><%=proxy.host%></host>
      <port><%=proxy.port%></port>
      <username><%=proxy.username%></username>
      <password><%=proxy.password%></password>
    </proxy>
  <% end %>
  </proxies>
</settings>
