# AUTOGENERATED BY THE GRADLE SCRIPT. DO NOT EDIT.

require 'jar_dependencies'
require_jar('com.microsoft.azure', 'azure-eventhubs-eph', '3.3.0')
require_jar('com.microsoft.azure', 'azure-eventhubs', '3.3.0')
require_jar('com.microsoft.azure', 'qpid-proton-j-extensions', '1.2.4')
require_jar('com.microsoft.azure', 'azure-storage', '8.6.6')
require_jar('com.microsoft.azure', 'azure-client-authentication', '1.7.3')
require_jar('com.microsoft.azure', 'azure-client-runtime', '1.7.3')
require_jar('com.microsoft.rest', 'client-runtime', '1.7.3')
require_jar('com.fasterxml.jackson.datatype', 'jackson-datatype-joda', '2.16.2')
require_jar('com.squareup.retrofit2', 'converter-jackson', '2.7.2')
require_jar('com.fasterxml.jackson.core', 'jackson-databind', '2.16.2')
require_jar('com.fasterxml.jackson.core', 'jackson-annotations', '2.16.2')
require_jar('com.fasterxml.jackson.core', 'jackson-core', '2.16.2')
require_jar('com.google.code.gson', 'gson', '2.8.9')
require_jar('com.microsoft.azure', 'azure-keyvault-core', '1.2.4')
require_jar('com.google.guava', 'guava', '32.0.1-jre')
require_jar('commons-codec', 'commons-codec', '1.14')
require_jar('com.nimbusds', 'nimbus-jose-jwt', '10.0.2')
require_jar('com.squareup.retrofit2', 'adapter-rxjava', '2.7.2')
require_jar('com.squareup.retrofit2', 'retrofit', '2.7.2')
require_jar('com.squareup.okhttp3', 'logging-interceptor', '3.12.2')
require_jar('com.squareup.okhttp3', 'okhttp-urlconnection', '3.12.2')
require_jar('com.squareup.okhttp3', 'okhttp', '4.12.0')
require_jar('org.apache.commons', 'commons-lang3', '3.18.0')
require_jar('com.squareup.okio', 'okio-jvm', '3.6.0')
require_jar('org.jetbrains.kotlin', 'kotlin-stdlib-jdk8', '1.9.10')
require_jar('org.jetbrains.kotlin', 'kotlin-stdlib-jdk7', '1.9.10')
require_jar('org.jetbrains.kotlin', 'kotlin-stdlib', '2.1.0')
require_jar('org.apache.qpid', 'proton-j', '0.33.8')
require_jar('org.slf4j', 'slf4j-api', '1.7.30')
require_jar('com.microsoft.azure', 'azure-annotations', '1.10.0')
require_jar('com.google.guava', 'failureaccess', '1.0.1')
require_jar('com.google.guava', 'listenablefuture', '9999.0-empty-to-avoid-conflict-with-guava')
require_jar('com.google.code.findbugs', 'jsr305', '3.0.2')
require_jar('org.checkerframework', 'checker-qual', '3.33.0')
require_jar('com.google.errorprone', 'error_prone_annotations', '2.18.0')
require_jar('com.google.j2objc', 'j2objc-annotations', '2.8')
require_jar('io.reactivex', 'rxjava', '1.3.8')
require_jar('joda-time', 'joda-time', '2.10.14')
require_jar('org.jetbrains', 'annotations', '13.0')
