# encoding: UTF-8

# This file contains data derived from the IANA Time Zone Database
# (https://www.iana.org/time-zones).

module TZInfo
  module Data
    module Definitions
      module Africa
        module Lubumbashi
          include TimezoneDefinition
          
          linked_timezone 'Africa/Lubumbashi', 'Africa/Maputo'
        end
      end
    end
  end
end
