# AUTOGENERATED BY THE GRADLE SCRIPT. DO NOT EDIT.

require 'jar_dependencies'
require_jar('io.confluent', 'kafka-avro-serializer', '7.9.1')
require_jar('org.apache.kafka', 'kafka-clients', '3.9.1')
require_jar('at.yawk.lz4', 'lz4-java', '1.10.1')
require_jar('io.confluent', 'kafka-schema-serializer', '7.9.1')
require_jar('io.confluent', 'kafka-schema-registry-client', '7.9.1')
require_jar('org.apache.avro', 'avro', '1.11.4')
require_jar('org.apache.commons', 'commons-compress', '1.26.2')
require_jar('com.google.guava', 'guava', '32.0.1-jre')
require_jar('io.confluent', 'logredactor', '1.0.12')
require_jar('io.confluent', 'common-utils', '7.9.1')
require_jar('com.github.luben', 'zstd-jni', '1.5.6-4')
require_jar('org.xerial.snappy', 'snappy-java', '1.1.10.5')
require_jar('org.slf4j', 'slf4j-api', '1.7.36')
require_jar('com.fasterxml.jackson.core', 'jackson-databind', '2.16.0')
require_jar('com.fasterxml.jackson.core', 'jackson-annotations', '2.16.0')
require_jar('com.fasterxml.jackson.dataformat', 'jackson-dataformat-csv', '2.16.0')
require_jar('com.fasterxml.jackson.core', 'jackson-core', '2.16.0')
require_jar('org.yaml', 'snakeyaml', '2.0')
require_jar('io.swagger.core.v3', 'swagger-annotations', '2.1.10')
require_jar('com.google.guava', 'failureaccess', '1.0.1')
require_jar('com.google.guava', 'listenablefuture', '9999.0-empty-to-avoid-conflict-with-guava')
require_jar('com.google.code.findbugs', 'jsr305', '3.0.2')
require_jar('org.checkerframework', 'checker-qual', '3.33.0')
require_jar('com.google.errorprone', 'error_prone_annotations', '2.18.0')
require_jar('com.google.j2objc', 'j2objc-annotations', '2.8')
require_jar('com.google.re2j', 're2j', '1.6')
require_jar('io.confluent', 'logredactor-metrics', '1.0.12')
require_jar('com.eclipsesource.minimal-json', 'minimal-json', '0.9.5')
require_jar('commons-codec', 'commons-codec', '1.17.0')
require_jar('commons-io', 'commons-io', '2.16.1')
require_jar('org.apache.commons', 'commons-lang3', '3.14.0')
